@@ -33,6 +33,7 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <unistd.h>  
 #include <stdio.h>
 #include <string.h>
@@ -41,10 +42,9 @@
 #include <sys/param.h>
 #include <fcntl.h>
 
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <eel/eel-vfs-extensions.h>
-
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #include "gedit-prefs-manager-app.h"
@@ -80,15 +80,20 @@ PROFILE (static GTimer *timer = NULL);
 struct _GeditDocumentPrivate
 {
 	gchar		    *uri;
-	gint 		     untitled_number;	
+	gint 		     untitled_number;
 
 	gchar		    *encoding;
 
+	gchar		    *mime_type;
+
 	gchar		    *last_searched_text;
 	gchar		    *last_replace_text;
 	gboolean  	     last_search_was_case_sensitive;
 	gboolean  	     last_search_was_entire_word;
 
+	gboolean             auto_save;
+	gint                 auto_save_interval;
+
 	guint		     auto_save_timeout;
 	gboolean	     last_save_was_manually; 
 
@@ -132,8 +137,7 @@ static gboolean	gedit_document_save_as_real (GeditDocument *doc, const gchar *ur
 					     gboolean create_backup_copy, GError **error);
 static void gedit_document_set_uri (GeditDocument* doc, const gchar* uri);
 
-static gboolean gedit_document_auto_save (GeditDocument *doc, GError **error);
-static gboolean gedit_document_auto_save_timeout (GeditDocument *doc);
+static gboolean gedit_document_auto_save (GeditDocument *doc);
 
 static GtkTextBufferClass *parent_class 	= NULL;
 static guint document_signals[LAST_SIGNAL] 	= { 0 };
@@ -308,6 +312,8 @@ gedit_document_init (GeditDocument *document)
 	document->priv->uri = NULL;
 	document->priv->untitled_number = 0;
 
+	document->priv->mime_type = g_strdup ("text/plain");
+
 	document->priv->readonly = FALSE;
 
 	document->priv->last_save_was_manually = TRUE;
@@ -322,7 +328,17 @@ gedit_document_init (GeditDocument *document)
 	gedit_document_set_max_undo_levels (document,
 					    gedit_prefs_manager_get_undo_actions_limit ());
 
-	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (document), FALSE);
+
+	/* TODO: Set the bracket matching tag style -- Paolo (10 Jan. 2005) */
+	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (document), 
+					      gedit_prefs_manager_get_bracket_matching ());
+
+	document->priv->auto_save = gedit_prefs_manager_get_auto_save ();
+	document->priv->auto_save = (document->priv->auto_save != FALSE);
+
+	document->priv->auto_save_interval = gedit_prefs_manager_get_auto_save_interval ();
+	if (document->priv->auto_save_interval <= 0)
+		document->priv->auto_save_interval = GPM_DEFAULT_AUTO_SAVE_INTERVAL;
 }
 
 static void
@@ -351,13 +367,18 @@ gedit_document_finalize (GObject *object)
 
 	if (document->priv->uri != NULL)
 	{
+		GtkTextIter iter;
 		gchar *position;
 		gchar *lang_id = NULL;
 		GtkSourceLanguage *lang;
 
+		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document),			
+				&iter,
+				gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document)));
+
 		position = g_strdup_printf ("%d", 
-					    gedit_document_get_cursor (document));
-		
+					    gtk_text_iter_get_offset (&iter));
+
 		gedit_metadata_manager_set (document->priv->uri,
 					    "position",
 					    position);
@@ -388,6 +409,7 @@ gedit_document_finalize (GObject *object)
 	}
 
 	g_free (document->priv->uri);
+	g_free (document->priv->mime_type);
 	g_free (document->priv->last_searched_text);
 	g_free (document->priv->last_replace_text);
 	g_free (document->priv->encoding);
@@ -461,18 +483,14 @@ gedit_document_new_with_uri (const gchar          *uri,
 void 		
 gedit_document_set_readonly (GeditDocument *document, gboolean readonly)
 {
-	gboolean auto_save;
-	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (document != NULL);
 	g_return_if_fail (document->priv != NULL);
 
-	auto_save = gedit_prefs_manager_get_auto_save ();
-	
 	if (readonly) 
 	{
-		if (auto_save && (document->priv->auto_save_timeout > 0))
+		if (document->priv->auto_save_timeout > 0)
 		{
 			gedit_debug (DEBUG_DOCUMENT, "Remove autosave timeout");
 
@@ -482,18 +500,15 @@ gedit_document_set_readonly (GeditDocument *document, gboolean readonly)
 	}
 	else
 	{
-		if (auto_save && (document->priv->auto_save_timeout <= 0))
+		if (document->priv->auto_save && 
+		    (document->priv->auto_save_timeout <= 0) && 
+                    !gedit_document_is_untitled (document))
 		{
-			gint auto_save_interval;
-			
 			gedit_debug (DEBUG_DOCUMENT, "Install autosave timeout");
 
-			auto_save_interval = 
-				gedit_prefs_manager_get_auto_save_interval ();
-				
 			document->priv->auto_save_timeout = g_timeout_add 
-				(auto_save_interval * 1000 * 60,
-		 		 (GSourceFunc)gedit_document_auto_save_timeout,
+				(document->priv->auto_save_interval * 1000 * 60,
+		 		 (GSourceFunc)gedit_document_auto_save,
 		  		 document);		
 		}
 	}
@@ -551,12 +566,19 @@ gedit_document_real_loaded (GeditDocument *document, const GError *error)
 	g_return_if_fail (document->priv->uri != NULL);
 
 	/* FIXME: commented since it does not work as expected - Paolo */
+	
 	/*
 	data = gedit_metadata_manager_get (document->priv->uri,
 					   "position");
 	if (data != NULL)
 	{
-		gedit_document_set_cursor (document, atoi (data));
+		GtkTextIter iter;
+
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document),
+						    &iter,
+						    atoi (data));
+
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (document), &iter);
 
 		g_free (data);
 	}
@@ -631,7 +653,7 @@ gedit_document_get_uri (const GeditDocument* doc)
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	if (doc->priv->uri == NULL)
-		return g_strdup_printf (_("%s %d"), _("Untitled"), doc->priv->untitled_number);
+		return g_strdup_printf (_("%s %d"), _("Unsaved Document"), doc->priv->untitled_number);
 	else
 	{
 		gchar *res;
@@ -649,13 +671,13 @@ gedit_document_get_short_name (const GeditDocument* doc)
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	if (doc->priv->uri == NULL)
-		return g_strdup_printf (_("%s %d"), _("Untitled"), doc->priv->untitled_number);
+		return g_strdup_printf (_("%s %d"), _("Unsaved Document"), doc->priv->untitled_number);
 	else
 	{
 		gchar *basename;
 		gchar *utf8_basename;
-		
-		basename = eel_uri_get_basename (doc->priv->uri);
+
+		basename = gedit_utils_uri_get_basename (doc->priv->uri);
 
 		if (basename != NULL) 
 		{
@@ -679,7 +701,7 @@ gedit_document_get_short_name (const GeditDocument* doc)
 			}
 			
 			/* there are problems */
-			utf8_basename = eel_make_valid_utf8 (basename);
+			utf8_basename = gedit_utils_make_valid_utf8 (basename);
 			g_free (basename);
 			return utf8_basename;
 		}
@@ -700,22 +722,34 @@ gedit_document_io_error_quark (void)
 }
 
 static gboolean
-gedit_document_auto_save_timeout (GeditDocument *doc)
+gedit_document_auto_save (GeditDocument *doc)
 {
+	const GeditEncoding *encoding;
+	gboolean create_backup;
 	GError *error = NULL;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 	g_return_val_if_fail (!gedit_document_is_readonly (doc), FALSE);
-
-	/* Remove timeout if now auto_save is FALSE */
-	if (!gedit_prefs_manager_get_auto_save ()) 
-		return FALSE;
+	g_return_val_if_fail (!gedit_document_is_untitled (doc), FALSE);
 
 	if (!gedit_document_get_modified (doc))
 		return TRUE;
 
-	gedit_document_auto_save (doc, &error);
+	encoding = gedit_document_get_encoding (doc);
+	create_backup = gedit_prefs_manager_get_create_backup_copy ();
+
+	/* we save a backup just if backups are enabled and if the last
+	 * save was manual.
+	 */
+	if (gedit_document_save_as_real (doc, doc->priv->uri, encoding,
+					 doc->priv->last_save_was_manually && create_backup,
+					 &error))
+	{
+		doc->priv->last_save_was_manually = FALSE;
+		g_get_current_time (&doc->priv->time_of_last_save_or_load);
+	}
 
 	if (error) 
 	{
@@ -728,27 +762,6 @@ gedit_document_auto_save_timeout (GeditDocument *doc)
 	return TRUE;
 }
 
-static gboolean
-gedit_document_auto_save (GeditDocument* doc, GError **error)
-{
-	const GeditEncoding *encoding;
-	
-	gedit_debug (DEBUG_DOCUMENT, "");
-	
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	encoding = gedit_document_get_encoding (doc);
-		
-	if (gedit_document_save_as_real (doc, doc->priv->uri, encoding,
-					 doc->priv->last_save_was_manually, NULL))
-	{
-		doc->priv->last_save_was_manually = FALSE;
-		g_get_current_time (&doc->priv->time_of_last_save_or_load);
-	}
-
-	return TRUE;
-}
-
 /* NOTE: thie function frees file_contents */
 static gboolean
 update_document_contents (GeditDocument        *doc, 
@@ -758,8 +771,6 @@ update_document_contents (GeditDocument        *doc,
 			  const GeditEncoding  *encoding,
 			  GError              **error)
 {
-	GtkTextIter iter, end;
-
 	PROFILE (
 		g_message ("Delete previous text %s: %.3f", uri, g_timer_elapsed (timer, NULL));
 	)
@@ -848,11 +859,8 @@ update_document_contents (GeditDocument        *doc,
 
 		gedit_document_begin_not_undoable_action (doc);
 
-		gedit_document_delete_text (doc, 0, -1);
-
 		/* Insert text in the buffer */
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
-		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, converted_text, len);
+		gtk_text_buffer_set_text (GTK_TEXT_BUFFER (doc), converted_text, len);
 
 		PROFILE (
 			g_message ("Text inserted: %.3f", g_timer_elapsed (timer, NULL));
@@ -865,16 +873,6 @@ update_document_contents (GeditDocument        *doc,
 			g_free (converted_text);
 		}
 
-		/* We had a newline in the buffer to begin with. (The buffer always contains
-   		 * a newline, so we delete to the end of the buffer to clean up. 
-		 * FIXME: Is this really needed? - Paolo (Jan 8, 2004) */
-		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
- 		gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc), &iter, &end);
-
-		/* Place the cursor at the start of the document or and old cursor position
-		 * if reverting */
-		gedit_document_set_cursor (doc, 0);
-
 		gedit_document_end_not_undoable_action (doc);
 
 		PROFILE (
@@ -1090,6 +1088,12 @@ get_info_cb (GnomeVFSAsyncHandle *handle,
 		return;
 	}
 
+	/* store the mime type */
+	g_free (doc->priv->mime_type);
+	doc->priv->mime_type = info_result->file_info->mime_type ?
+			       g_strdup (info_result->file_info->mime_type) :
+			       g_strdup ("text/plain");
+
 	if (info_result->file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
 	{
 		doc->priv->temp_size = (gulong)info_result->file_info->size;
@@ -1131,6 +1135,8 @@ load_local (GeditDocument *doc)
 	res = gnome_vfs_get_file_info (doc->priv->temp_uri,
 				       info,
 				       GNOME_VFS_FILE_INFO_DEFAULT |
+				       GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+				       GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
 				       GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 
 	if (res != GNOME_VFS_OK)
@@ -1181,6 +1187,12 @@ load_local (GeditDocument *doc)
 		return FALSE;
 	}
 
+	/* store the mime type */
+	g_free (doc->priv->mime_type);
+	doc->priv->mime_type = info->mime_type ?
+			       g_strdup (info->mime_type) :
+			       g_strdup ("text/plain");
+
 	gnome_vfs_file_info_unref (info);
 
 	res = gnome_vfs_read_entire_file (doc->priv->temp_uri, 
@@ -1331,6 +1343,8 @@ gedit_document_load (GeditDocument        *doc,
 		gnome_vfs_async_get_file_info (&doc->priv->info_handle,
 					       uri_list,
 					       GNOME_VFS_FILE_INFO_DEFAULT |
+					       GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+					       GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
 					       GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					       GNOME_VFS_PRIORITY_MAX,
 					       get_info_cb,
@@ -1366,7 +1380,7 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 		const GeditEncoding *encoding;
 		gchar *converted_text;
 
-		if (gedit_encoding_get_current () != gedit_encoding_get_utf8 ())
+		if (gedit_encoding_get_current () == gedit_encoding_get_utf8 ())
 		{
 			if (g_utf8_validate (stdin_data, -1, NULL))
 			{
@@ -1448,34 +1462,10 @@ gedit_document_is_untouched (const GeditDocument *doc)
 		(!gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)));
 }
 
-/* FIXME: Remove this function when gnome-vfs will add an equivalent public
-   function - Paolo (Mar 05, 2004) */
-static gchar *
-get_slow_mime_type (const char *text_uri)
-{
-	GnomeVFSFileInfo *info;
-	char *mime_type;
-	GnomeVFSResult result;
-
-	info = gnome_vfs_file_info_new ();
-	result = gnome_vfs_get_file_info (text_uri, info,
-					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-					  GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
-					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-	if (info->mime_type == NULL || result != GNOME_VFS_OK) {
-		mime_type = NULL;
-	} else {
-		mime_type = g_strdup (info->mime_type);
-	}
-	gnome_vfs_file_info_unref (info);
-
-	return mime_type;
-}
-
 static void
 gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 {
-	GtkSourceLanguage *language;
+	GtkSourceLanguage *language = NULL;
 	gchar *data;
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
@@ -1503,52 +1493,36 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 	{
 		gedit_debug (DEBUG_DOCUMENT, "Language: %s", data);
 
-		if (strcmp (data, "_NORMAL_") == 0)
-			language = NULL;
-		else
+		if (strcmp (data, "_NORMAL_") != 0)
 			language = gedit_languages_manager_get_language_from_id (
 						gedit_get_languages_manager (),
 						data);
 
-		gedit_document_set_language (doc, language);
-
 		g_free (data);
 	}
 	else
 	{
-		gchar *mime_type;
-
-		mime_type = get_slow_mime_type (uri);
-	
-		if (mime_type != NULL)
+		if (doc->priv->mime_type != NULL)
 		{
-			GtkSourceLanguage *language;
-			
-			gedit_debug (DEBUG_DOCUMENT, "MIME-TYPE: %s", mime_type);
-
 			language = gtk_source_languages_manager_get_language_from_mime_type (
 						gedit_get_languages_manager (),
-						mime_type);
-	
-			gedit_document_set_language (doc, language);
-	
-			g_free (mime_type);
+						doc->priv->mime_type);
 		}
 		else
 		{
-			g_warning ("Couldn't get mime type for file `%s'", uri);
+			/* should never happen */
+			g_warning ("gedit document %s has NULL mime_type", uri);
 		}
-
-		
 	}
 
+	gedit_document_set_language (doc, language);
+
 	g_signal_emit (G_OBJECT (doc), document_signals[NAME_CHANGED], 0);
 }
 	
 gboolean 
 gedit_document_save (GeditDocument *doc, GError **error)
 {	
-	gboolean auto_save;
 	gboolean create_backup_copy;
 	const GeditEncoding *encoding;
 	
@@ -1564,16 +1538,12 @@ gedit_document_save (GeditDocument *doc, GError **error)
 	encoding = gedit_document_get_encoding (doc);
 	g_return_val_if_fail (encoding != NULL, FALSE);
 	
-	auto_save = gedit_prefs_manager_get_auto_save ();
 	create_backup_copy = gedit_prefs_manager_get_create_backup_copy ();
 		
-	if (auto_save) 
+	if (doc->priv->auto_save_timeout > 0)
 	{
-		if (doc->priv->auto_save_timeout > 0)
-		{
-			g_source_remove (doc->priv->auto_save_timeout);
-			doc->priv->auto_save_timeout = 0;
-		}
+		g_source_remove (doc->priv->auto_save_timeout);
+		doc->priv->auto_save_timeout = 0;
 	}
 	
 	ret = gedit_document_save_as_real (doc, 
@@ -1588,13 +1558,10 @@ gedit_document_save (GeditDocument *doc, GError **error)
 		g_get_current_time (&doc->priv->time_of_last_save_or_load);
 	}
 
-	if (auto_save && (doc->priv->auto_save_timeout <= 0)) 
+	if (doc->priv->auto_save && (doc->priv->auto_save_timeout <= 0)) 
 	{
-		gint auto_save_interval = 
-			gedit_prefs_manager_get_auto_save_interval ();
-
 		doc->priv->auto_save_timeout =
-			g_timeout_add (auto_save_interval * 1000 * 60,
+			g_timeout_add (doc->priv->auto_save_interval * 1000 * 60,
 				       (GSourceFunc) gedit_document_auto_save, 
 				       doc);
 	}
@@ -1606,10 +1573,10 @@ gboolean
 gedit_document_save_as (GeditDocument* doc, const gchar *uri, 
 			const GeditEncoding *encoding, GError **error)
 {	
-	gboolean auto_save;
-	
+	gboolean create_backup;
+
 	gboolean ret = FALSE;
-	
+
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
@@ -1619,19 +1586,15 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri,
 	if (encoding == NULL)
 		encoding = gedit_document_get_encoding (doc);
 
-	auto_save = gedit_prefs_manager_get_auto_save ();
+	create_backup = gedit_prefs_manager_get_create_backup_copy ();
 
-	if (auto_save) 
+	if (doc->priv->auto_save_timeout > 0)
 	{
-
-		if (doc->priv->auto_save_timeout > 0)
-		{
-			g_source_remove (doc->priv->auto_save_timeout);
-			doc->priv->auto_save_timeout = 0;
-		}
+		g_source_remove (doc->priv->auto_save_timeout);
+		doc->priv->auto_save_timeout = 0;
 	}
 
-	if (gedit_document_save_as_real (doc, uri, encoding, TRUE, error))
+	if (gedit_document_save_as_real (doc, uri, encoding, create_backup, error))
 	{
 		gedit_document_set_uri (doc, uri);
 		gedit_document_set_readonly (doc, FALSE);
@@ -1642,42 +1605,14 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri,
 		gedit_document_set_encoding (doc, encoding);
 			
 		ret = TRUE;
-	}		
-
-	if (auto_save && (doc->priv->auto_save_timeout <= 0)) 
-	{
-		gint auto_save_interval =
-			gedit_prefs_manager_get_auto_save_interval ();
-		
-		doc->priv->auto_save_timeout =
-			g_timeout_add (auto_save_interval * 1000 * 60,
-				       (GSourceFunc)gedit_document_auto_save, doc);
 	}
 
-	return ret;
-}
-
-gboolean	
-gedit_document_save_a_copy_as (GeditDocument *doc, const gchar *uri, 
-			       const GeditEncoding *encoding, GError **error)
-{
-	gboolean m;
-	gboolean ret;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (doc->priv != NULL, FALSE);
-	g_return_val_if_fail (uri != NULL, FALSE);
-
-	if (encoding == NULL)
-		encoding = gedit_document_get_encoding (doc);
-	
-	m = gedit_document_get_modified (doc);
-
-	ret = gedit_document_save_as_real (doc, uri, encoding, FALSE, error);
-	
-	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), m);	 
+	if (doc->priv->auto_save && (doc->priv->auto_save_timeout <= 0)) 
+	{
+		doc->priv->auto_save_timeout =
+			g_timeout_add (doc->priv->auto_save_interval * 1000 * 60,
+				       (GSourceFunc)gedit_document_auto_save, doc);
+	}
 
 	return ret;
 }
@@ -1773,10 +1708,33 @@ follow_symlinks (const gchar *filename, GError **error)
 	return NULL;
 }
 
+/* FIXME: Remove this function when gnome-vfs will add an equivalent public
+   function - Paolo (Mar 05, 2004) */
+static gchar *
+get_slow_mime_type (const char *text_uri)
+{
+	GnomeVFSFileInfo *info;
+	char *mime_type;
+	GnomeVFSResult result;
+
+	info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info (text_uri, info,
+					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+					  GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
+					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (info->mime_type == NULL || result != GNOME_VFS_OK) {
+		mime_type = NULL;
+	} else {
+		mime_type = g_strdup (info->mime_type);
+	}
+	gnome_vfs_file_info_unref (info);
+
+	return mime_type;
+}
+
 /* FIXME: define new ERROR_CODE and remove the error 
  * strings from here -- Paolo
  */
-
 static gboolean	
 gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEncoding *encoding,
 			     gboolean create_backup_copy, GError **error)
@@ -1790,11 +1748,15 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 	mode_t saved_umask;
 	struct stat st;
 	gchar *chars = NULL;
-	gint chars_len;
 	gint fd;
 	gint retval;
+	gsize chars_len = 0;
+	gsize new_len = 0;
 	gboolean res;
 	gboolean add_cr;
+	GtkTextIter start_iter;
+	GtkTextIter end_iter;
+	gchar *detected_mime;
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
@@ -1818,7 +1780,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 		gchar *scheme_string;
 
 		gchar *temp = gnome_vfs_get_uri_scheme (uri);
-                scheme_string = eel_make_valid_utf8 (temp);
+		scheme_string = gedit_utils_make_valid_utf8 (temp);
 		g_free (temp);
 
 		if (scheme_string != NULL)
@@ -1910,7 +1872,18 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 		goto out;
 	}
 
-	chars = gedit_document_get_chars (doc, 0, -1);
+	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (doc), &start_iter, &end_iter);
+	chars = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc),
+					   &start_iter, &end_iter, TRUE);
+
+	chars_len = strlen (chars);
+
+	add_cr = FALSE;
+	
+	if (chars_len >= 1)
+	{
+		add_cr = (*(chars + chars_len - 1) != '\n');
+	}
 	
 	if (encoding != gedit_encoding_get_utf8 ())
 	{
@@ -1918,8 +1891,9 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 		gchar* converted_file_contents = NULL;
 
 		converted_file_contents = gedit_convert_from_utf8 (chars, 
-								   -1, 
+								   chars_len, 
 								   encoding,
+								   &new_len,
 								   &conv_error);
 
 		if (conv_error != NULL)
@@ -1937,17 +1911,42 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 			chars = converted_file_contents;
 		}
 	}
-
-	chars_len = strlen (chars);
-
-	add_cr = (*(chars + chars_len - 1) != '\n');
+	else
+	{
+		new_len = chars_len;
+	}
 
 	/* Save the file content */
-	res = (write (fd, chars, chars_len) == chars_len);
+	res = (write (fd, chars, new_len) == new_len);
 
 	if (res && add_cr)
+	{
 		/* Add \n if needed */
-		res = (write (fd, "\n", 1) == 1);
+		if (encoding != gedit_encoding_get_utf8 ())
+		{
+			gchar* converted_n = NULL;
+		
+			converted_n = gedit_convert_from_utf8 ("\n", 
+							       -1, 
+							       encoding,
+							       &new_len,
+							       NULL);
+			
+			if (converted_n == NULL)
+			{
+				g_warning ("Cannot add '\\n' at the end of the file.");
+			}
+			else
+			{
+				res = (write (fd, converted_n, new_len) == new_len);
+				g_free (converted_n);
+			}
+		}
+		else
+		{	
+			res = (write (fd, "\n", 1) == 1);
+		}
+	}
 
 	if (!res)
 	{
@@ -2038,6 +2037,11 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
 
+	/* update the mime type */
+	detected_mime = get_slow_mime_type (uri);
+	g_free (doc->priv->mime_type);
+	doc->priv->mime_type = detected_mime ? detected_mime : g_strdup ("text/plain");
+
 	retval = TRUE;
 
 	/* Done */
@@ -2054,6 +2058,19 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 	return retval;
 }
 
+const gchar *
+gedit_document_get_mime_type (const GeditDocument* doc)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (doc != NULL, FALSE);
+	g_return_val_if_fail (doc->priv != NULL, FALSE);
+
+	g_return_val_if_fail (doc->priv->mime_type != NULL, "text/plain");
+
+	return doc->priv->mime_type;
+}
+
 gboolean	
 gedit_document_is_untitled (const GeditDocument* doc)
 {
@@ -2098,44 +2115,6 @@ gedit_document_get_deleted (GeditDocument *doc)
 	return deleted;
 }
 
-/**
- * gedit_document_get_char_count:
- * @doc: a #GeditDocument 
- * 
- * Gets the number of characters in the buffer; note that characters
- * and bytes are not the same, you can't e.g. expect the contents of
- * the buffer in string form to be this many bytes long. 
- * 
- * Return value: number of characters in the document
- **/
-gint 
-gedit_document_get_char_count (const GeditDocument *doc)
-{
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	return gtk_text_buffer_get_char_count (GTK_TEXT_BUFFER (doc));
-}
-
-/**
- * gedit_document_get_line_count:
- * @doc: a #GeditDocument 
- * 	
- * Obtains the number of lines in the buffer. 
- * 
- * Return value: number of lines in the document
- **/
-gint 
-gedit_document_get_line_count (const GeditDocument *doc)
-{
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	return gtk_text_buffer_get_line_count (GTK_TEXT_BUFFER (doc));
-}
-
 void 
 gedit_document_revert (GeditDocument *doc)
 {
@@ -2166,62 +2145,6 @@ gedit_document_insert_text (GeditDocument *doc, gint pos, const gchar *text, gin
 	gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, text, len);
 }
 
-void 
-gedit_document_insert_text_at_cursor (GeditDocument *doc, const gchar *text, gint len)
-{
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	g_return_if_fail (text != NULL);
-	g_return_if_fail (g_utf8_validate (text, len, NULL));
-
-	gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (doc), text, len);
-}
-
-void 
-gedit_document_delete_text (GeditDocument *doc, gint start, gint end)
-{
-	GtkTextIter start_iter;
-	GtkTextIter end_iter;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	g_return_if_fail (start >= 0);
-	g_return_if_fail ((end > start) || end < 0);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
-
-	if (end < 0)
-		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
-	else
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
-
-	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc), &start_iter, &end_iter);
-}
-
-gchar*
-gedit_document_get_chars (GeditDocument *doc, gint start, gint end)
-{
-	GtkTextIter start_iter;
-	GtkTextIter end_iter;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
-	g_return_val_if_fail (start >= 0, NULL);
-	g_return_val_if_fail ((end > start) || (end < 0), NULL);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
-
-	if (end < 0)
-		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
-	else
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
-
-	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start_iter, &end_iter, TRUE);
-}
-
 void
 gedit_document_set_max_undo_levels (GeditDocument *doc, gint max_undo_levels)
 {
@@ -2583,32 +2506,6 @@ gedit_document_find_prev (GeditDocument* doc, gint flags)
 	return gedit_document_find_again (doc, flags);
 }
 
-gboolean 
-gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
-{
-	gboolean ret;
-	GtkTextIter iter;
-	GtkTextIter sel_bound;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
-
-	ret = gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),			
-						    &iter,
-						    &sel_bound);
-
-	gtk_text_iter_order (&iter, &sel_bound);	
-
-	if (start != NULL)
-		*start = gtk_text_iter_get_offset (&iter); 
-
-	if (end != NULL)
-		*end = gtk_text_iter_get_offset (&sel_bound); 
-
-	return ret;
-}
-
 void
 gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 {
@@ -2684,75 +2581,6 @@ gedit_document_replace_all (GeditDocument *doc,
 	return cont;
 }
 
-guint
-gedit_document_get_line_at_offset (const GeditDocument *doc, guint offset)
-{
-	GtkTextIter iter;
-	
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), 0);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, offset);
-	
-	return gtk_text_iter_get_line (&iter);
-}
-
-gint 
-gedit_document_get_cursor (GeditDocument *doc)
-{
-	GtkTextIter iter;
-	
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), 0);
-
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-			&iter,
-			gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
-
-	return gtk_text_iter_get_offset (&iter); 
-}
-
-void
-gedit_document_set_cursor (GeditDocument *doc, gint cursor)
-{
-	GtkTextIter iter;
-	
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	
-	/* Place the cursor at the requested position */
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, cursor);
-	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
-}
-
-void
-gedit_document_set_selection (GeditDocument *doc, gint start, gint end)
-{
- 	GtkTextIter start_iter;
-	GtkTextIter end_iter;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	g_return_if_fail (start >= 0);
-	g_return_if_fail ((end > start) || (end < 0));
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
-
-	if (end < 0)
-		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
-	else
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
-
-	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &end_iter);
-
-	gtk_text_buffer_move_mark_by_name (GTK_TEXT_BUFFER (doc),
-					"selection_bound", &start_iter);
-}
-
 void 
 gedit_document_set_language (GeditDocument *doc, GtkSourceLanguage *lang)
 {
@@ -2833,3 +2661,79 @@ gedit_document_get_seconds_since_last_save_or_load (GeditDocument *doc)
 	return (current_time.tv_sec - doc->priv->time_of_last_save_or_load.tv_sec);
 }
 
+void
+gedit_document_enable_auto_save (GeditDocument *doc, gboolean enable)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	enable = (enable != FALSE);
+
+	if (doc->priv->auto_save == enable)
+		return;
+
+	doc->priv->auto_save = enable;
+
+	if (doc->priv->auto_save && 
+	    (doc->priv->auto_save_timeout <= 0) && 
+	    !gedit_document_is_untitled (doc) &&
+	    !gedit_document_is_readonly (doc))
+	{
+		gedit_debug (DEBUG_DOCUMENT, "Install autosave timeout");
+
+		doc->priv->auto_save_timeout = g_timeout_add 
+							(doc->priv->auto_save_interval * 1000 * 60,
+			 		 		 (GSourceFunc)gedit_document_auto_save,	
+					  		 doc);		
+
+		return;
+	}
+	
+	if (!doc->priv->auto_save && (doc->priv->auto_save_timeout > 0))
+	{
+		gedit_debug (DEBUG_DOCUMENT, "Remove autosave timeout");
+
+		g_source_remove (doc->priv->auto_save_timeout);
+		doc->priv->auto_save_timeout = 0;
+
+		return;
+	}
+
+	g_return_if_fail ((!doc->priv->auto_save && (doc->priv->auto_save_timeout <= 0)) || 
+			  gedit_document_is_untitled (doc) || gedit_document_is_readonly (doc));
+}
+
+void
+gedit_document_set_auto_save_interval (GeditDocument *doc, gint interval)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (interval > 0);
+
+	if (doc->priv->auto_save_interval == interval)
+		return;
+	
+	doc->priv->auto_save_interval = interval;
+		
+	if (!doc->priv->auto_save)
+		return;
+
+	if (doc->priv->auto_save_timeout > 0)
+	{
+		gedit_debug (DEBUG_DOCUMENT, "Remove autosave timeout");
+
+		g_source_remove (doc->priv->auto_save_timeout);
+	
+		gedit_debug (DEBUG_DOCUMENT, "Install new autosave timeout");
+
+		g_return_if_fail (!gedit_document_is_untitled (doc));
+		g_return_if_fail (!gedit_document_is_readonly (doc));
+
+		doc->priv->auto_save_timeout = g_timeout_add 
+						(doc->priv->auto_save_interval * 1000 * 60,
+		 		 		 (GSourceFunc)gedit_document_auto_save,	
+				  		 doc);		
+	}
+}
