@@ -33,14 +33,9 @@
 #include <config.h>
 #endif
 
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include <eel/eel-vfs-extensions.h>
-#include <eel/eel-alert-dialog.h>
-#include <eel/eel-string.h>
-
 #include "gedit2.h"
 #include "gedit-file.h"
 #include "gedit-debug.h"
@@ -49,6 +44,7 @@
 #include "gedit-mdi.h"
 #include "gedit-recent.h" 
 #include "gedit-file-selector-util.h"
+#include "gedit-io-error-dialogs.h"
 #include "gedit-plugins-engine.h"
 #include "recent-files/egg-recent-model.h"
 #include "gedit-prefs-manager-app.h"
@@ -64,11 +60,6 @@
 
 PROFILE (static GTimer *timer = NULL);
 
-static gchar 	*get_dirname_from_uri 		(const char          *uri);
-
-static gboolean  gedit_file_save_as_real 	(const gchar         *file_name, 
-						 const GeditEncoding *encoding,
-						 GeditMDIChild       *child);
 static void	 open_files 			(void);
 static void	 show_progress_bar 		(gchar               *uri, 
 						 gboolean             show, 
@@ -81,6 +72,12 @@ static void 	 document_loading_cb 		(GeditDocument       *document,
 
 static gchar* gedit_default_path = NULL;
 
+/* type of the elements of the uris_to_open list */
+typedef struct {
+	gchar *uri;
+	gboolean create;
+} UriToLoad;
+
 /* Global variables for managing async loading */
 static gint                 num_of_uris_to_open = 0;
 static gint                 opened_uris 	= 0;
@@ -94,28 +91,6 @@ static gint		    times_called	= 0;
 static GtkWidget           *loading_window	= NULL;
 static GtkWidget           *progress_bar	= NULL;
 
-static gchar *
-get_dirname_from_uri (const char *uri)
-{
-	GnomeVFSURI *vfs_uri;
-	gchar *name;
-	gchar *res;
-
-	/* Make VFS version of URI. */
-	vfs_uri = gnome_vfs_uri_new (uri);
-	if (vfs_uri == NULL) {
-		return NULL;
-	}
-
-	/* Extract name part. */
-	name = gnome_vfs_uri_extract_dirname (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);
-
-	res = g_strdup_printf ("file:///%s", name);
-	g_free (name);
-
-	return res;
-}
 
 void 
 gedit_file_new (void)
@@ -138,29 +113,30 @@ gedit_file_new (void)
 	g_return_if_fail (ret != FALSE);
 	gedit_debug (DEBUG_COMMANDS, "View added.");
 	
-	gtk_widget_grab_focus (GTK_WIDGET (gedit_get_active_view ()));
+	gtk_widget_grab_focus (gedit_get_active_view ());
 }
 
-void 
+gboolean
 gedit_file_close (GtkWidget *view)
 {
 	BonoboMDIChild* child;
+	gboolean closed;
 
 	gedit_debug (DEBUG_FILE, "");
 
-	g_return_if_fail (view != NULL);
+	g_return_val_if_fail (view != NULL, FALSE);
 
 	child = bonobo_mdi_get_child_from_view (view);
-	g_return_if_fail (child != NULL);
+	g_return_val_if_fail (child != NULL, FALSE);
 
 	if (g_list_length (bonobo_mdi_child_get_views (child)) > 1)
 	{		
-		bonobo_mdi_remove_view (BONOBO_MDI (gedit_mdi), view, FALSE);
+		closed = bonobo_mdi_remove_view (BONOBO_MDI (gedit_mdi), view, FALSE);
 		gedit_debug (DEBUG_COMMANDS, "View removed.");
 	}
 	else
 	{
-		bonobo_mdi_remove_child (BONOBO_MDI (gedit_mdi), child, FALSE);
+		closed = bonobo_mdi_remove_child (BONOBO_MDI (gedit_mdi), child, FALSE);
 		gedit_debug (DEBUG_COMMANDS, "Child removed.");
 	}
 
@@ -172,6 +148,8 @@ gedit_file_close (GtkWidget *view)
 	}
 
 	gedit_debug (DEBUG_FILE, "END");
+
+	return closed;
 }
 
 void
@@ -197,7 +175,7 @@ gedit_file_open (GeditMDIChild *active_child)
 
 		if ((raw_uri != NULL) && gedit_utils_uri_has_file_scheme (raw_uri))
 		{
-			default_path = get_dirname_from_uri (raw_uri);
+			default_path = gedit_utils_uri_get_dirname (raw_uri);
 		}
 
 		g_free (raw_uri);
@@ -223,18 +201,18 @@ gedit_file_open (GeditMDIChild *active_child)
 gboolean 
 gedit_file_save (GeditMDIChild* child, gboolean force)
 {
-	gint ret;
+	gboolean ret;
 	GeditDocument* doc = NULL;
 	GError *error = NULL;
 	gchar *uri = NULL;
-	
+
 	gedit_debug (DEBUG_FILE, "");
 
 	g_return_val_if_fail (child != NULL, FALSE);
 	
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
-	
+
 	if (gedit_document_is_untitled (doc))
 	{
 		gedit_debug (DEBUG_FILE, "Untitled");
@@ -258,22 +236,22 @@ gedit_file_save (GeditMDIChild* child, gboolean force)
 				deleted = !gedit_utils_uri_exists (raw_uri);
 		}
 		g_free (raw_uri);
-			
+
 		if (!deleted)
 			return TRUE;
 	}
-	
+
 	uri = gedit_document_get_uri (doc);
 	g_return_val_if_fail (uri != NULL, FALSE);
-	
+
 	gedit_utils_flash_va (_("Saving document \"%s\"..."), uri);	
-	
+
 	ret = gedit_document_save (doc, &error);
 
 	if (!ret)
 	{
 		GtkWidget *view;
-		
+
 		g_return_val_if_fail (error != NULL, FALSE);
 		gedit_debug (DEBUG_FILE, "FAILED");
 
@@ -288,49 +266,40 @@ gedit_file_save (GeditMDIChild* child, gboolean force)
 
 			bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
 		}
-		
-		gedit_utils_error_reporting_saving_file (uri, error,
-					GTK_WINDOW (gedit_get_active_window ()));
+
+		gedit_error_reporting_saving_file (uri,
+						   error,
+						   GTK_WINDOW (gedit_get_active_window ()));
 
 		g_error_free (error);
 
 		gedit_utils_flash_va (_("The document \"%s\" has not been saved."), uri);
-
-		g_free (uri);
-
-		return FALSE;
 	}	
 	else
 	{
-		EggRecentModel *recent;
-		EggRecentItem *item;
 		gchar *raw_uri;
-		
+
 		gedit_debug (DEBUG_FILE, "OK");
 
 		gedit_utils_flash_va (_("The document \"%s\" has been saved."), uri);
 
-		g_free (uri);
-
 		raw_uri = gedit_document_get_raw_uri (doc);
 		g_return_val_if_fail (raw_uri != NULL, TRUE);
-		
-		recent = gedit_recent_get_model ();
-		item = egg_recent_item_new_from_uri (raw_uri);
-		egg_recent_item_add_group (item, "gedit");
-		egg_recent_model_add_full (recent, item);
-		egg_recent_item_unref (item);
+
+		gedit_recent_add (raw_uri);
 
 		g_free (raw_uri);
-		
-		return TRUE;
 	}
+
+	g_free (uri);
+
+	return ret;
 }
 
 gboolean
 gedit_file_save_as (GeditMDIChild *child)
 {
-	gchar *file;
+	gchar *file_uri;
 	gboolean ret = FALSE;
 	GeditDocument *doc;
 	GtkWidget *view;
@@ -348,6 +317,7 @@ gedit_file_save_as (GeditMDIChild *child)
 
 	view = GTK_WIDGET (g_list_nth_data (
 			bonobo_mdi_child_get_views (BONOBO_MDI_CHILD (child)), 0));
+
 	if (view != NULL)
 	{
 		GtkWindow *window;
@@ -357,7 +327,7 @@ gedit_file_save_as (GeditMDIChild *child)
 
 		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
 	}
-	
+
 	if (gedit_document_is_untitled (doc))
 	{
 		path = (gedit_default_path != NULL) ? 
@@ -372,15 +342,14 @@ gedit_file_save_as (GeditMDIChild *child)
 	else
 	{
 		gchar *raw_uri = gedit_document_get_raw_uri (doc);
-
 		g_return_val_if_fail (raw_uri != NULL, FALSE);
 
 		if (gedit_utils_uri_has_file_scheme (raw_uri))
 		{
-			fname = eel_uri_get_basename (raw_uri);
+			fname = gedit_utils_uri_get_basename (raw_uri);
 			g_return_val_if_fail (fname != NULL, FALSE);
 
-			path = get_dirname_from_uri (raw_uri);
+			path = gedit_utils_uri_get_dirname (raw_uri);
 		}
 		else
 		{
@@ -396,7 +365,7 @@ gedit_file_save_as (GeditMDIChild *child)
 
 	encoding = gedit_document_get_encoding (doc);
 
-	file = gedit_file_selector_save (
+	file_uri = gedit_file_selector_save (
 			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 			FALSE,
 		        _("Save as..."), 
@@ -404,110 +373,64 @@ gedit_file_save_as (GeditMDIChild *child)
 			fname,
 			untitled_name,
 			&encoding);
-	
+
 	g_free (fname);
 	g_free (untitled_name);
 	g_free (path);
 
-	if (file != NULL) 
+	if (file_uri != NULL) 
 	{
 		gchar *uri;
 		gchar *file_utf8;
+		GError *error = NULL;
 
-		uri = eel_make_uri_from_shell_arg (file);
-		g_return_val_if_fail (uri != NULL, FALSE);
-
-		file_utf8 = eel_format_uri_for_display (uri);
+		file_utf8 = gnome_vfs_format_uri_for_display (file_uri);
 		if (file_utf8 != NULL)
 			gedit_utils_flash_va (_("Saving document \"%s\"..."), file_utf8);
-		
-		ret = gedit_file_save_as_real (uri, encoding, child);
-		
-		if (ret)
+
+		uri = gnome_vfs_make_uri_canonical (file_uri);
+		g_return_val_if_fail (uri != NULL, FALSE);
+
+		ret = gedit_document_save_as (doc, uri, encoding, &error);
+
+		if (!ret)
+		{
+			g_return_val_if_fail (error != NULL, FALSE);
+
+			gedit_error_reporting_saving_file (uri,
+							   error,
+							   GTK_WINDOW (gedit_get_active_window ()));
+
+			g_error_free (error);
+
+			if (file_utf8 != NULL)
+				gedit_utils_flash_va (_("The document \"%s\" has not been saved."),
+						      file_utf8);
+		}
+		else
 		{			
 			if (gedit_default_path != NULL)
 				g_free (gedit_default_path);
 
-			gedit_default_path = get_dirname_from_uri (file);
+			gedit_default_path = gedit_utils_uri_get_dirname (file_uri);
+
+			gedit_recent_add (uri);
 
 			if (file_utf8 != NULL)
-			{
 				gedit_utils_flash_va (_("The document \"%s\" has been saved."), 
 						      file_utf8);
-			}
-		}
-		else
-		{
-			if (file_utf8 != NULL)
-			{
-				gedit_utils_flash_va (_("The document \"%s\" has not been saved."),
-						      file_utf8);
-			}
 		}
 
-		gedit_debug (DEBUG_FILE, "File: %s", file);
+		gedit_debug (DEBUG_FILE, "File: %s", file_uri);
+
 		g_free (uri);
-		g_free (file);
-
-		if (file_utf8 != NULL)
-			g_free (file_utf8);
-		
+		g_free (file_uri);
+		g_free (file_utf8);
 	}
 
 	return ret;
 }
 
-static gboolean
-gedit_file_save_as_real (const gchar* file_name, const GeditEncoding *encoding, GeditMDIChild *child)
-{
-	gchar *uri;
-	gboolean ret;
-	GeditDocument *doc = NULL;
-	GError *error = NULL;
-
-	gedit_debug (DEBUG_FILE, "");
-
-	g_return_val_if_fail (child != NULL, FALSE);
-
-	doc = child->document;
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	uri = eel_make_uri_canonical (file_name);
-	g_return_val_if_fail (uri != NULL, FALSE);
-	
-	ret = gedit_document_save_as (doc, uri, encoding, &error);
-
-	if (!ret)
-	{
-		g_return_val_if_fail (error != NULL, FALSE);
-
-		gedit_utils_error_reporting_saving_file (file_name, error,
-					GTK_WINDOW (gedit_get_active_window ()));
-
-		g_error_free (error);
-		
-		g_free (uri);
-
-		return FALSE;
-		
-	}	
-	else
-	{
-		EggRecentModel *recent;
-		EggRecentItem *item;
-
-		recent = gedit_recent_get_model ();
-		item = egg_recent_item_new_from_uri (uri);
-		egg_recent_item_add_group (item, "gedit");
-		egg_recent_model_add_full (recent, item);
-		egg_recent_item_unref (item);
-
-		g_free (uri);
-
-		return TRUE;
-	}
-}
-
 gboolean
 gedit_file_close_all (void)
 {
@@ -561,12 +484,12 @@ void
 gedit_file_save_all (void)
 {
 	guint i = 0;
-	GeditMDIChild* child;
-	GtkWidget* view;
+	GeditMDIChild *child;
+	GtkWidget *view;
 
 	gedit_debug (DEBUG_FILE, "");
 
-	view = bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi));
+	view = gedit_get_active_view ();
 
 	for (i = 0; i < g_list_length (bonobo_mdi_get_children (BONOBO_MDI (gedit_mdi))); i++)
 	{
@@ -576,7 +499,7 @@ gedit_file_save_all (void)
 		gedit_file_save (child, FALSE);	
 	}
 
-	if (view != bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)))
+	if (view != gedit_get_active_view ())
 	{
 		GtkWindow *window;
 
@@ -591,8 +514,7 @@ gedit_file_save_all (void)
 static gboolean
 gedit_file_revert_dialog (GeditDocument *doc)
 {
-	GtkWidget *msgbox;
-	AtkObject *obj;
+	GtkWidget *dialog;
 	gint ret;
 	gchar *name;
 	gchar *primary_msg;
@@ -670,7 +592,7 @@ gedit_file_revert_dialog (GeditDocument *doc)
 		hours = seconds / 3600;
 
 		secondary_msg = g_strdup_printf (
-					ngettext ("Changes made to the document in the last %d hour "
+					ngettext ("Changes made to the document in the last hour "
 					    	  "will be permanently lost.",
 						  "Changes made to the document in the last %d hours "
 					    	  "will be permanently lost.",
@@ -678,34 +600,28 @@ gedit_file_revert_dialog (GeditDocument *doc)
 					hours);
 	}
 
-	msgbox = eel_alert_dialog_new (GTK_WINDOW (gedit_get_active_window ()),
-				       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				       GTK_MESSAGE_QUESTION,
-				       GTK_BUTTONS_NONE,
-				       primary_msg,
-				       secondary_msg, 
-				       NULL);
+	dialog = gtk_message_dialog_new (GTK_WINDOW (gedit_get_active_window ()),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 primary_msg);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  secondary_msg);
 	g_free (primary_msg);
 	g_free (secondary_msg);
 
-	/* Add Cancel button */
-	gtk_dialog_add_button (GTK_DIALOG (msgbox), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
-	/* Add Revert button */
-	gedit_dialog_add_button (GTK_DIALOG (msgbox), 
+	gedit_dialog_add_button (GTK_DIALOG (dialog), 
 			_("_Revert"), GTK_STOCK_REVERT_TO_SAVED, GTK_RESPONSE_YES);
 
-	gtk_dialog_set_default_response	(GTK_DIALOG (msgbox), GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_default_response	(GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
-	obj = gtk_widget_get_accessible (msgbox);
-	if (GTK_IS_ACCESSIBLE (obj))
-		atk_object_set_name (obj, _("Question"));
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
-	gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
+	ret = gtk_dialog_run (GTK_DIALOG (dialog));
 
-	ret = gtk_dialog_run (GTK_DIALOG (msgbox));
-		
-	gtk_widget_destroy (msgbox);
+	gtk_widget_destroy (dialog);
 
 	return (ret == GTK_RESPONSE_YES);
 }
@@ -746,8 +662,9 @@ document_reverted_cb (GeditDocument *document,
 
 		gedit_debug (DEBUG_DOCUMENT, "Error reverting file %s", uri_to_display);
 
-		gedit_utils_error_reporting_reverting_file (raw_uri, error,
-					GTK_WINDOW (gedit_get_active_window ()));
+		gedit_error_reporting_reverting_file (raw_uri,
+						      error,
+						      GTK_WINDOW (gedit_get_active_window ()));
 
 		gedit_utils_flash_va (_("The document \"%s\" has not been reverted."), uri_to_display);
 	}
@@ -771,28 +688,28 @@ gedit_file_revert (GeditMDIChild *child)
 	gedit_debug (DEBUG_FILE, "");
 
 	g_return_if_fail (child != NULL);
-	
+
 	doc = child->document;
 	g_return_if_fail (doc != NULL);
-	
+
 	if (!gedit_file_revert_dialog (doc))
 		return;
-	
+
 	uri = gedit_document_get_uri (doc);
 	g_return_if_fail (uri != NULL);
-	
+
 	gedit_utils_flash_va (_("Reverting the document \"%s\"..."), uri);	
-	
+	g_free (uri);
+
 	g_signal_connect (G_OBJECT (doc),
 			  "loading",
 			  G_CALLBACK (document_loading_cb),
 			  (gpointer)TRUE);
-				  
 	g_signal_connect (G_OBJECT (doc),
 			  "loaded",
 			  G_CALLBACK (document_reverted_cb),
 			  NULL);
-	
+
 	gedit_document_revert (doc);
 }
 
@@ -800,7 +717,7 @@ gboolean
 gedit_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
 {
 	gboolean ret = FALSE;
-	GeditView* active_view;
+	GtkWidget *active_view;
 	gchar *uri_utf8;
 
 	if (gedit_mdi_get_state (gedit_mdi) != GEDIT_STATE_NORMAL)
@@ -814,23 +731,19 @@ gedit_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
 
 	/* FIXME */
 	ret = gedit_file_open_single_uri (uri_utf8, NULL);
-	
-	if (!ret) 
+
+	if (!ret)
 	{
-		EggRecentModel *model;
 		gchar *uri;
 
 		uri = egg_recent_item_get_uri (item);
-
-		model = gedit_recent_get_model ();
-		egg_recent_model_delete (model, uri);
-
+		gedit_recent_remove (uri);
 		g_free (uri);
 	}
-		
+
 	active_view = gedit_get_active_view ();
 	if (active_view != NULL)
-		gtk_widget_grab_focus (GTK_WIDGET (active_view));
+		gtk_widget_grab_focus (active_view);
 
 	g_free (uri_utf8);
 
@@ -853,10 +766,10 @@ gedit_file_open_single_uri (const gchar* uri, const GeditEncoding *encoding)
 	if (uri == NULL) 
 		return FALSE;
 
-	full_path = eel_make_uri_from_input (uri);
+	full_path = gnome_vfs_make_uri_from_input (uri);
 
 	uri_list = g_slist_prepend (NULL, full_path);
-	
+
 	gedit_file_open_uri_list (uri_list, encoding, 0, FALSE);
 
 	return TRUE;
@@ -940,70 +853,6 @@ gedit_file_open_from_stdin (GeditMDIChild *active_child)
 	return ret;
 }
 
-
-static void 
-create_new_file (const gchar *uri)
-{
-	gint ret;
-	GeditMDIChild *new_child;
-
-	new_child = gedit_mdi_child_new_with_uri (uri, NULL);
-	ret = bonobo_mdi_add_child (BONOBO_MDI (gedit_mdi), BONOBO_MDI_CHILD (new_child));
-	g_return_if_fail (ret != FALSE);
-	gedit_debug (DEBUG_COMMANDS, "Child added.");
-
-	ret = bonobo_mdi_add_view (BONOBO_MDI (gedit_mdi), BONOBO_MDI_CHILD (new_child));
-	g_return_if_fail (ret != FALSE);
-	gedit_debug (DEBUG_COMMANDS, "View added.");
-
-	gtk_widget_grab_focus (GTK_WIDGET (gedit_get_active_view ()));
-}
-
-static gboolean
-update_recent_files_real (const gchar *uri)
-{
-	EggRecentModel *recent;
-	EggRecentItem  *item;
-
-	recent = gedit_recent_get_model ();
-	item = egg_recent_item_new_from_uri (uri);
-	egg_recent_item_add_group (item, "gedit");
-	egg_recent_model_add_full (recent, item);
-	egg_recent_item_unref (item);
-
-	return FALSE;
-}
-
-static void
-update_recent_files (const gchar *uri)
-{
-	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-			 (GSourceFunc)update_recent_files_real,
-			 g_strdup (uri),
-			 (GDestroyNotify)g_free);
-}
-
-static gboolean
-remove_recent_file_real (const gchar *uri)
-{
-	EggRecentModel *recent;
-
-	recent = gedit_recent_get_model ();
-	egg_recent_model_delete (recent, uri);
-
-	return FALSE;
-}
-
-static void
-remove_recent_file (const gchar *uri)
-{
-	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-			 (GSourceFunc)remove_recent_file_real,
-			 g_strdup (uri),
-			 (GDestroyNotify)g_free);
-}
-
-
 #define MAX_URI_IN_DIALOG_LENGTH 50
 
 static void
@@ -1014,6 +863,7 @@ show_loading_dialog (GtkWindow *parent, gchar *uri, gboolean reverting)
 	GtkWidget *image;
 	GtkWidget *vbox;
 	GtkWidget *label;
+	GtkWidget *uri_label;
 	gchar *str;
 	gchar *uri_for_display;
 	gchar *full_formatted_uri;
@@ -1024,9 +874,9 @@ show_loading_dialog (GtkWindow *parent, gchar *uri, gboolean reverting)
 
 	full_formatted_uri = gnome_vfs_format_uri_for_display (uri);
 
-	/* Truncate the URI so it doesn't get insanely wide.  */
-        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
-						   MAX_URI_IN_DIALOG_LENGTH);
+	/* Truncate the URI so it doesn't get insanely wide. */
+        uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
+							   MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
 	g_return_if_fail (uri_for_display != NULL);
@@ -1038,6 +888,9 @@ show_loading_dialog (GtkWindow *parent, gchar *uri, gboolean reverting)
 	gtk_window_set_position (GTK_WINDOW (loading_window), GTK_WIN_POS_CENTER_ON_PARENT);
 		
 	gtk_window_set_decorated (GTK_WINDOW (loading_window), FALSE); 
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (loading_window), TRUE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (loading_window), TRUE);
+
 	gtk_window_set_transient_for (GTK_WINDOW (loading_window), parent);
 
 	frame = gtk_frame_new (NULL);
@@ -1054,24 +907,24 @@ show_loading_dialog (GtkWindow *parent, gchar *uri, gboolean reverting)
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
 
-	if (reverting)
-	{
-		str = g_strdup_printf ("<b>%s</b>\n%s", _("Reverting file:"), uri_for_display);
-	}
-	else
-	{
-		str = g_strdup_printf ("<b>%s</b>\n%s", _("Loading file:"), uri_for_display);		
-	}
-	
+	str = g_strdup_printf ("<b>%s</b>",
+			reverting? _("Reverting file:") : _("Loading file:"));
+
 	label = gtk_label_new (str);
-	g_free (uri_for_display);
 	g_free (str);
-	
+
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 
+	uri_label = gtk_label_new (uri_for_display);
+	g_free (uri_for_display);
+
+	gtk_box_pack_start (GTK_BOX (vbox), uri_label, FALSE, FALSE, 0);
+	gtk_label_set_justify (GTK_LABEL (uri_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (uri_label), 0, 0.5);
+
 	progress_bar = gtk_progress_bar_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), progress_bar, FALSE, FALSE, 0);
 
@@ -1141,11 +994,14 @@ document_loading_cb (GeditDocument *document,
 		     gboolean       reverting)
 {
 	static GTimer *d_timer = NULL;
-	
+	UriToLoad *utl;
 	gchar *uri;
 	double et;
 
-	uri = (gchar*)uris_to_open->data;
+	utl = (UriToLoad *)uris_to_open->data;
+	g_return_if_fail (utl != NULL);
+
+	uri = utl->uri;
 	g_return_if_fail (uri != NULL);
 
 	if (d_timer == NULL)
@@ -1202,10 +1058,14 @@ document_loaded_cb (GeditDocument *document,
 		    const GError  *error,
 		    GeditMDIChild *new_child)
 {
+	UriToLoad *utl;
 	gchar *uri;
 	gchar *uri_to_display;
-	
-	uri = (gchar*)uris_to_open->data;
+
+	utl = (UriToLoad *)uris_to_open->data;
+	g_return_if_fail (utl != NULL);
+
+	uri = utl->uri;
 	g_return_if_fail (uri != NULL);
 
 	show_progress_bar (NULL, FALSE, FALSE);
@@ -1232,15 +1092,15 @@ document_loaded_cb (GeditDocument *document,
 
 		gedit_debug (DEBUG_DOCUMENT, "Error loading file %s", uri_to_display);
 
-		remove_recent_file (uri);
+		gedit_recent_remove (uri);
 
-		gedit_utils_error_reporting_loading_file (uri, encoding_to_use, (GError *)error,
-					GTK_WINDOW (gedit_get_active_window ()));
+		gedit_error_reporting_loading_file (uri,
+						    encoding_to_use,
+						    (GError *)error,
+						    GTK_WINDOW (gedit_get_active_window ()));
 
 		if (new_child != NULL)
-		{
 			children_to_unref = g_slist_prepend (children_to_unref, new_child);
-		}
 
 		gedit_debug (DEBUG_DOCUMENT, "Returning %s", uri_to_display);
 	}
@@ -1265,7 +1125,7 @@ document_loaded_cb (GeditDocument *document,
 		{
 			gchar *default_path;
 			
-			default_path = get_dirname_from_uri (uri);
+			default_path = gedit_utils_uri_get_dirname (uri);
 
 			if (default_path != NULL)
 			{
@@ -1279,83 +1139,144 @@ document_loaded_cb (GeditDocument *document,
 			g_message ("Updated gedit_default_path: %.3f", g_timer_elapsed (timer, NULL));
 		)
 
-		update_recent_files (uri);	
+		gedit_recent_add (uri);	
 	}
 
 	g_free (uri_to_display);
-	
+
 	/* Load the next file in the uris_to_open list */
 	g_free (uri);
+	g_free (utl);
 	uris_to_open = g_slist_delete_link (uris_to_open, uris_to_open);
 
 	open_files ();
 }
 
+static gboolean
+unref_real (GSList *children)
+{
+	g_slist_foreach (children, (GFunc)g_object_unref, NULL);
+
+	return FALSE;
+}
+
+static void
+unref_not_opened_children ()
+{
+	GSList *children;
+
+	if (children_to_unref == NULL)
+		return;
+
+	children = g_slist_copy (children_to_unref);
+
+	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+			 (GSourceFunc)unref_real,
+			 children,
+			 (GDestroyNotify)g_slist_free);
+}
+
+static void
+open_files_done ()
+{
+	/* Add all the views in the right order */
+	new_children = g_slist_reverse (new_children);
+
+	PROFILE (
+		g_message ("Document Loaded: %.3f", g_timer_elapsed (timer, NULL));
+	)
+
+	bonobo_mdi_add_views (BONOBO_MDI (gedit_mdi),
+			      new_children);
+
+	PROFILE (
+		g_message ("View added: %.3f", g_timer_elapsed (timer, NULL));
+	)
+
+	g_slist_free (new_children);
+	new_children = NULL;
+
+	/* Unref the children that were not opened to an error */
+	unref_not_opened_children ();
+	g_slist_free (children_to_unref);
+	children_to_unref = NULL;
+
+	if (num_of_uris_to_open > 1)
+	{
+		gedit_utils_set_status (NULL);
+		gedit_utils_flash_va (ngettext("Loaded %d file",
+					       "Loaded %d files", 
+					       opened_uris), 
+				      opened_uris);
+	}
+
+	/* Clean up temp variables */
+	num_of_uris_to_open = 0;
+	opened_uris = 0;
+	encoding_to_use = NULL;
+	line = -1;
+
+	PROFILE (
+		g_message ("Done all: %.3f", g_timer_elapsed (timer, NULL));
+		g_timer_destroy (timer);
+	)
+
+	gedit_mdi_set_state (gedit_mdi, GEDIT_STATE_NORMAL);
+
+	return;
+}
+
 static void
 open_files ()
 {
 	GeditDocument  *active_document;
-	
-	gchar          *uri;
-	gchar          *uri_to_display;
+	UriToLoad *utl;
+	gchar *uri;
+	gchar *uri_to_display;
 
 	if (uris_to_open == NULL)
-	{		
-		/* Add all the views in the right order */
-		new_children = g_slist_reverse (new_children);
-
-		PROFILE (
-			g_message ("Document Loaded: %.3f", g_timer_elapsed (timer, NULL));
-		)
-		
-		bonobo_mdi_add_views (BONOBO_MDI (gedit_mdi),
-				      new_children);
-	
-		PROFILE (
-			g_message ("View added: %.3f", g_timer_elapsed (timer, NULL));
-		)
-	
-		g_slist_free (new_children);
-		new_children = NULL;
-
-		/* Unref the children that were not opened to an error */
-		g_slist_foreach (children_to_unref, (GFunc)g_object_unref, NULL);
-		g_slist_free (children_to_unref);
-		children_to_unref = NULL;
-		
-		if (num_of_uris_to_open > 1)
-		{
-			gedit_utils_set_status (NULL);
-			gedit_utils_flash_va (ngettext("Loaded %d file",
-						       "Loaded %d files", 
-						       opened_uris), 
-					      opened_uris);
-		}
-
-		/* Clean up temp variables */
-		num_of_uris_to_open = 0;
-		opened_uris = 0;
-		encoding_to_use = NULL;
-		line = -1;
-
-		PROFILE (
-			g_message ("Done all: %.3f", g_timer_elapsed (timer, NULL));
-			g_timer_destroy (timer);
-		)
-
-		gedit_mdi_set_state (gedit_mdi, GEDIT_STATE_NORMAL);
+	{
+		/* stop recursion */
+		open_files_done ();
 		return;
 	}
-	
-	uri = (gchar*)uris_to_open->data;
+
+	utl = (UriToLoad *)uris_to_open->data;
+	g_return_if_fail (utl != NULL);
+
+	uri = utl->uri;
 	g_return_if_fail (uri != NULL);
-	
+
 	gedit_debug (DEBUG_FILE, "File name: %s", uri);
 
+	uri_to_display = gnome_vfs_format_uri_for_display (uri);
+
+	/* create children for non existing files */
+	if (utl->create)
+	{
+		GeditMDIChild *new_child;
+
+		new_child = gedit_mdi_child_new_with_uri (uri, NULL);
+		bonobo_mdi_add_child (BONOBO_MDI (gedit_mdi), BONOBO_MDI_CHILD (new_child));
+
+		new_children = g_slist_prepend (new_children, new_child);
+		opened_uris++;
+
+		gedit_utils_set_status (NULL);
+		gedit_utils_flash_va (_("Created file \"%s\""), uri_to_display);
+
+		g_free (uri_to_display);
+		g_free (uri);
+		g_free (utl);
+		uris_to_open = g_slist_delete_link (uris_to_open, uris_to_open);
+
+		open_files ();
+
+		return;
+	}
+
 	active_document = gedit_get_active_document ();
 	
-	uri_to_display = gnome_vfs_format_uri_for_display (uri);
-
 	PROFILE (
 		g_message ("URI to open: %.3f", g_timer_elapsed (timer, NULL));
 	)
@@ -1384,14 +1305,16 @@ open_files ()
 		if (new_child == NULL)
 		{
 			/* FIXME: this is a too generic error message - Paolo */
-			gedit_utils_error_reporting_loading_file (uri, encoding_to_use, NULL,
-					GTK_WINDOW (gedit_get_active_window ()));
+			gedit_error_reporting_loading_file (uri,
+							    encoding_to_use,
+							    NULL,
+							    GTK_WINDOW (gedit_get_active_window ()));
 
 			gedit_utils_flash_va (_("Error loading file \"%s\""), uri_to_display);
 
 			g_free (uri_to_display);
 			g_free (uri);
-			 
+			g_free (utl);
 			uris_to_open = g_slist_delete_link (uris_to_open, uris_to_open);
 
 			open_files ();
@@ -1445,8 +1368,10 @@ gedit_file_open_uri_list (GSList *uri_list,
 			  gint line_pos,
 			  gboolean create)
 {
+	GSList *l;
+
 	gedit_debug (DEBUG_FILE, "");
-	
+
 	if (uri_list == NULL)
 		return TRUE;
 
@@ -1468,35 +1393,37 @@ gedit_file_open_uri_list (GSList *uri_list,
 		timer = g_timer_new ();
 	)
 
-	while (uri_list != NULL)
+	for (l = uri_list; l; l = l->next)
 	{
 		gchar *uri;
+		UriToLoad *u;
+
+		uri = gnome_vfs_make_uri_canonical ((const gchar*)l->data);
 
-		uri = gnome_vfs_make_uri_canonical ((const gchar*)uri_list->data);
-		
 		gedit_debug (DEBUG_FILE, "URI: %s", uri);
 
-		if (uri != NULL)
+		if (!uri)
 		{
-			if (!gedit_utils_uri_has_file_scheme (uri) || 
-			     gedit_utils_uri_exists (uri))
-			{
-				uris_to_open = g_slist_prepend (uris_to_open, uri);
-			}
-			else
-			{
-				if (create)
-				{
-					create_new_file (uri);
-				}
-				else
-				{
-					uris_to_open = g_slist_prepend (uris_to_open, uri);
-				}
-			}		
-		}	
-			
-		uri_list = g_slist_next (uri_list);
+			g_warning ("Could not get canonical uri\n");
+			continue;
+		}
+
+		u = g_new (UriToLoad, 1);
+
+		u->uri = uri;
+
+		if (create &&
+		    gedit_utils_uri_has_file_scheme (uri) &&
+		    !gedit_utils_uri_exists (uri))
+		{
+			u->create = TRUE;
+		}
+		else
+		{
+			u->create = FALSE;
+		}
+
+		uris_to_open = g_slist_prepend (uris_to_open, u);
 	}
 
 	uris_to_open = g_slist_reverse (uris_to_open);
